/**********************************************************************
zyGrib: meteorological GRIB file viewer
Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
***********************************************************************/

#include <QMessageBox>
#include <cmath>
#include <cassert>

#include "DialogServerStatus.h"
#include "Util.h"
#include "Version.h"

//-------------------------------------------------------------------------------
DialogServerStatus::DialogServerStatus 
		(QNetworkAccessManager *manager, MeteoDataModel meteoDataModel, QWidget *parent)
: DialogBoxBase (parent)
{
	QString page;
    host = "www.zygrib.org";
	downloadError = false;
	reply_step1 = NULL;
	
	switch (meteoDataModel) {
		case NOAA_GFS :
			page = "/noaa/getGfsRunLog.php";
			break;
		case MB_NMM4 :
			page = "/noaa/getmblueinfo.php?model=nmm4";
			break;
		default:
			return;			// ??????
	}
	
    setWindowTitle (tr("Server"));
    QFrame *ftmp;
    QLabel *label;
    frameGui = createFrameGui(this);
    
    layout = new QGridLayout(this);
    int lig=0;
    //-------------------------
    lig ++;
    QFont font;
    font.setBold(true);
    label = new QLabel(tr("File server status"), this);
    label->setFont(font);
    layout->addWidget( label,    lig,0, 1,-1, Qt::AlignCenter);
    lig ++;
    ftmp = new QFrame(this); ftmp->setFrameShape(QFrame::HLine); layout->addWidget( ftmp, lig,0, 1, -1);
    //-------------------------
    lig ++;
    layout->addWidget( frameGui,  lig,0,   1, 2);
    //-------------------------
    lig ++;
    ftmp = new QFrame(this); ftmp->setFrameShape(QFrame::HLine); layout->addWidget( ftmp, lig,0, 1, -1);
    //-------------------------
    lig ++;
    btOK     = new QPushButton(tr("Ok"), this);
    layout->addWidget( btOK,    lig,0);

	//===============================================================
    connect(btOK, SIGNAL(clicked()), this, SLOT(slotBtOK()));
    //===============================================================
    timeLoad.start();
	
	QNetworkRequest request;
	request.setUrl (QUrl("http://"+host+page) );
	request.setRawHeader ("User-Agent", 
							qPrintable(Version::getAppName()+"/"+Version::getVersion()) );

	reply_step1 = manager->get (request);
	connect (reply_step1, SIGNAL(downloadProgress (qint64,qint64)), 
					this, SLOT(downloadProgress_step1 (qint64,qint64)));
	connect (reply_step1, SIGNAL(error(QNetworkReply::NetworkError)),
					this, SLOT(slotNetworkError (QNetworkReply::NetworkError)));
}
//-------------------------------------------------------------------------------
DialogServerStatus::~DialogServerStatus() {
	if (reply_step1) {
		reply_step1->deleteLater ();
		reply_step1 = NULL;
	}
}

//-------------------------------------------------------------------------------
void DialogServerStatus::slotNetworkError (QNetworkReply::NetworkError /*err*/)
{
	if (!downloadError) {
		downloadError = true;
		errorMessage = reply_step1->errorString();
		QMessageBox::critical (this, tr("Error"), errorMessage);
	}
}	

//-------------------------------------------------------------------------------
void DialogServerStatus::downloadProgress_step1 (qint64 done, qint64 total)
{
	QString strDur; 
    int tp = timeLoad.elapsed();
    QTextStream(&strDur) << tp << " ms";
	
	if (downloadError) {
        lbResponseTime->setText(strDur);
        lbResponseStatus->setText(tr("Error"));
        lbMessage->setText("Error: "+errorMessage);
    }
    else if (done == total)
    {
        lbResponseTime->setText(strDur);
        lbResponseStatus->setText(tr("ok"));
        
		QByteArray data = reply_step1->readAll ();
		QHash <QString,QString> allNoaa = readData (data);
        
        lbRunDate->setText (getData (allNoaa, "run_date"));
        lbRunHour->setText (getData (allNoaa, "run_hour"));
        lbCurrentJob->setText (getData (allNoaa, "current_job"));
		lbClient->setText (getData (allNoaa, "client"));
		
		lbMessage->setText (allNoaa.value ("message"));
    }
}

//-------------------------------------------------------------------------------
QString DialogServerStatus::getData (const QHash <QString,QString> &data, const QString &key)
{
    QString rep = data.value (key);
    if (rep == "")
		rep=tr("invalid format");
	return rep;
}

//-------------------------------------------------------------------------------
QHash <QString,QString> DialogServerStatus::readData (const QByteArray &data)
{
	QHash <QString,QString> result;
	QString tmp;
	QString strbuf (data);
	QStringList lsbuf = strbuf.split("\n");
    for (int i=0; i < lsbuf.size(); i++)
    {
        QStringList lsval = lsbuf.at(i).split(":");
        if (lsval.size() > 1) {
            if ( lsval.at(0).endsWith("run_date")) {
                tmp = lsval.at(1).trimmed();
                if (tmp.size()==8) {   // format: 20080523
                    QDateTime dt= QDateTime::fromString(tmp+"0000","yyyyMMddHHmm");
                    if (dt.isValid()) {
                        //rep = Util::formatDateLong(dt.toTime_t(),false);
						result.insert ("run_date", dt.toString("yyyy-MM-dd"));
                    }
                }
            }
            if ( lsval.at(0).endsWith("run_hour")) {
				result.insert ("run_hour", lsval.at(1).trimmed()+" h UTC");
            }
            if (lsval.at(0) == "client") {
				result.insert ("client", lsval.at(1).trimmed());
            }
            if (lsval.at(0) == "current_job") {
				result.insert ("current_job", lsval.at(1).trimmed());
            }
            if (lsval.at(0) == "message") {
				result.insert ("message", lsval.at(1).trimmed());
            }
        }
    }
	return result;
}
//-------------------------------------------------------------------------------
void DialogServerStatus::slotBtOK()
{
    accept();
}
//-------------------------------------------------------------------------------
void DialogServerStatus::slotBtCancel()
{
    reject();
}

//=============================================================================
// GUI
//=============================================================================
QFrame *DialogServerStatus::createFrameGui(QWidget *parent)
{
    QFrame * frm = new QFrame(parent);
    QFrame * ftmp;
    QLabel * label;
    QGridLayout  *lay = new QGridLayout(frm);
    int lig=0;
    //-------------------------
    lig ++;
    label = new QLabel(tr("Connection :"), frm);
    lay->addWidget( label,    lig,0, Qt::AlignRight);
    lbResponseStatus = new QLabel("", frm);
    lay->addWidget( lbResponseStatus, lig,1, Qt::AlignLeft);
    //-------------------------
    lig ++;
    label = new QLabel(tr("Response time :"), frm);
    lay->addWidget( label,    lig,0, Qt::AlignRight);
    lbResponseTime = new QLabel("", frm);
    lay->addWidget( lbResponseTime, lig,1, Qt::AlignLeft);
    //-------------------------
    lig ++;
    ftmp = new QFrame(this); ftmp->setFrameShape(QFrame::HLine); lay->addWidget( ftmp, lig,0, 1, -1);
    //-------------------------
    lig ++;
    label = new QLabel(tr("Forecast date :"), frm);
    lay->addWidget( label,    lig,0, Qt::AlignRight);
    lbRunDate = new QLabel("", frm);
    lay->addWidget( lbRunDate, lig,1, Qt::AlignLeft);
    //-------------------------
    lig ++;
    label = new QLabel(tr("Hour :"), frm);
    lay->addWidget( label,    lig,0, Qt::AlignRight);
    lbRunHour = new QLabel("", frm);
    lay->addWidget( lbRunHour, lig,1, Qt::AlignLeft);
    //-------------------------
    lig ++;
    label = new QLabel(tr("Current job :"), frm);
    lay->addWidget( label,    lig,0, Qt::AlignRight);
    lbCurrentJob = new QLabel("", frm);
    lay->addWidget( lbCurrentJob, lig,1, Qt::AlignLeft);
    //-------------------------
    lig ++;
    ftmp = new QFrame(this); ftmp->setFrameShape(QFrame::HLine); lay->addWidget( ftmp, lig,0, 1, -1);
    //-------------------------
    lig ++;
    lbClient = new QLabel("", frm);
    lay->addWidget( lbClient, lig,0,1,2, Qt::AlignLeft);
    //-------------------------
    lig ++;
    lbMessage = new QLabel("", frm);
    lay->addWidget( lbMessage, lig,0,1,2, Qt::AlignLeft);

    return frm;
}











