/*
 *  CurveDrawer.cpp
 *  zyGrib
 *
 *  Created by Tim Holtschneider on 29.05.10.
 *  Copyright 2010. All rights reserved.
 *
 */
#include <QMessageBox>
#include "CurveDrawer.h"

#include "Orthodromie.h"
#include "BoatSpeed.h"

//-------------------------------------------------------------------------------
// CurveDrawer: Constructor for single point
//-------------------------------------------------------------------------------
CurveDrawer::CurveDrawer( GriddedPlotter *plotter, 
						  double lon, double lat, QWidget *parentWindow)
	: QDialog(parentWindow)
{		
	this->plotter	= plotter;
	this->lonStart	= lon;
	this->latStart	= lat;
	this->lonEnd    = 999.9;
	this->latEnd	= 999.9;
	if( !privateConstructor() )
		return;

	// display initial plot of wind speeds
	if( createTable( GRB_WIND_XY2D ) ){
		initDataPlot( GRB_WIND_XY2D );
		initInterface();
	}
}
//-------------------------------------------------------------------------------
// CurveDrawer: Constructor for two POIs
//-------------------------------------------------------------------------------
CurveDrawer::CurveDrawer( GriddedPlotter *plotter, 
						  double lonStart, double latStart, double lonEnd, double latEnd, QWidget *parentWindow)
: QDialog(parentWindow)
{	
	this->plotter	= plotter;
	this->lonStart	= lonStart;
	this->latStart	= latStart;
	this->lonEnd	= lonEnd;
	this->latEnd	= latEnd;
	if( !privateConstructor() )
		return;
	
	// display initial plot of wind speeds
	if( createTablePOI( GRB_WIND_XY2D ) ){
		initDataPlot( GRB_WIND_XY2D );
		initInterface();
	}
}
//-------------------------------------------------------------------------------
CurveDrawer::~CurveDrawer()
{
	for( int i=0; i<4; i++ )
		if( qwtCurve[i] ) { delete qwtCurve[i]; qwtCurve[i] = NULL; }
	for( int i=0; i<2; i++ ) {
		if( qwtMarker[i] ) { delete qwtMarker[i]; qwtMarker[i] = NULL; }
	}
	if( qwtDataPlot ) {delete qwtDataPlot; qwtDataPlot = NULL;};
}
//-------------------------------------------------------------------------------
// create data tables for single point
//-------------------------------------------------------------------------------
bool CurveDrawer::createTable( const int &iDataChosen ) 
{
	bool bFirst;
	float dDir, dDirH, dStep;
	dDir = 0; dDirH = 0; bFirst = true;
	time_t tCurDateRec = reader->getClosestDateFromNow();
	
	if( !qvY1Axis.isEmpty() )		{qvY1Axis.clear();};
	if( !qvDates.isEmpty() )		{qvDates.clear();};
	if( !qvHoursFromNow.isEmpty() )	{qvHoursFromNow.clear();};
	if( !qvWindDir.isEmpty() )		{qvWindDir.clear();};
	if( !qvWindDev.isEmpty() )		{qvWindDev.clear();};	

	
	//int iNumDates = reader->getNumberOfDates();
	std::set<time_t> sdates = reader->getListDates();
	std::set<time_t>::iterator iter;
	// calculate step size
	dStep = (*(sdates.rbegin()) - *(sdates.begin()))/3600. / sdates.size();
	
	//-----------------------------------------------
	// Titre 1 : une colonne par date+horaires
	//-----------------------------------------------
	for (iter=sdates.begin(), iDataCnt=0; iter!=sdates.end(); iter++, iDataCnt++)
	{
		time_t date = *iter;
		// Grib data for this point and this date
		DataPointInfo *pinfo = new DataPointInfo( reader, lonStart,latStart, date );

		// calculate date points for x axis
		qvHoursFromNow << (double) ((date - tCurDateRec)/3600.);
		QDateTime dt = Util::applyTimeZone( date );
		qvDates << dt.toString("dd.MM.yy hh.mm");
		
		// evaluate data points for y axis to the left
		qvY1Axis << pinfo->getData( iDataChosen );
		if( iDataChosen==GRB_TEMP ) {
			qvTempMax << pinfo->getData( GRB_TMAX );
			qvTempMin << pinfo->getData( GRB_TMIN );
		}
		
		// evaluate data points for y axis to the right, currently fixed to wind direction
		// and derivative of wind direction
		dDirH = pinfo->windDir;
		qvWindDir << pinfo->windDir;

		// avoid peak in derivative of first data point
		if( bFirst ) { bFirst = false; dDir = dDirH; }
		
		// mangle 360 degree periodicity
		if( (dDirH - dDir) > 180 ) {
			qvWindDev << ((360 - (dDirH - dDir)) / dStep);
		} else if( (dDirH - dDir) < -180 ) {
			qvWindDev << ((360 + (dDirH - dDir)) / dStep);
		} else {
			qvWindDev << ( (dDirH - dDir) / dStep );
		}
		dDir = dDirH;
		delete pinfo;
	}
		
	// set initial display
	qvXAxis  = &qvHoursFromNow;
	qvY2Axis = &qvWindDir;
	return true;
}
//-------------------------------------------------------------------------------
// create data tables for POIs
//-------------------------------------------------------------------------------
bool CurveDrawer::createTablePOI( const int &iDataChosen ) 
{
	bool bFirst;
	double dDir, dDirH, tmpLon, tmpLat;
	int iZero = 0;
	dDir = 0; dDirH = 0; bFirst = true; tmpLon = 0; tmpLat = 0;
	time_t tCurDateRec = reader->getClosestDateFromNow();
	time_t tLastDate = *reader->getListDates().rbegin();
	
	if( !qvY1Axis.isEmpty() )			{qvY1Axis.clear();};
	if( !qvDates.isEmpty() )		{qvDates.clear();};
	if( !qvHoursFromNow.isEmpty() )	{qvHoursFromNow.clear();};
	if( !qvWindDir.isEmpty() )		{qvWindDir.clear();};
	if( !qvWindDev.isEmpty() )		{qvWindDev.clear();};

	// get initial course and distance between the two POIs
	Orthodromie *myOrtho = new Orthodromie( lonStart, latStart, lonEnd, latEnd );

	time_t dateTimeStep = reader->getClosestDateFromNow();
	time_t dateDataStep = reader->getClosestDateFromNow();

	double px = lonStart;
	double py = latStart;
	double myAngle = myOrtho->getAzimutDeg();
	// fixed time step size of 1 hour
	double dDist = 0, dDistStep = 6., dTimeStep = 1.;
	double dRemain;
	
	BoatSpeed *myBoatSpeed = new BoatSpeed();
	dRemain = myOrtho->getDistance();
	// loop while end of data is not jet reached and remaining distance > 1 mile
	while( dRemain > 1. && dateDataStep < tLastDate ) {

		DataPointInfo *pinfo;
		dRemain = myOrtho->getDistance();
		
		if( reader->getFirstRecord()->isPointInMap(px, py) ) {
			
			// get data point info at a valid date
			pinfo = new DataPointInfo( reader, px, py, dateDataStep );

			// start plot at current time		
			qvHoursFromNow << (double) ((dateTimeStep - tCurDateRec) / 3600.);
			qvDist  << dDist;

			// get way made good in one hour under current wind conditions
			dDistStep = myBoatSpeed->getBoatSpeed( pinfo->windDir, pinfo->windSpeed, myAngle  );
			if( dDistStep <= 0 ) {
				iZero += 1;
				if (iZero >= 5 ) {
					QMessageBox::warning(this, tr("Warning"),tr("No sufficient wind to travel waypoint route!"));
					delete this;
					return false;
				}
				dDistStep = 0.00001;
			}
			// if end point is close by adjust step size in distance
			if( dDistStep > myOrtho->getDistance() ) {
				dDistStep = myOrtho->getDistance();
			}			
			dDist   += dDistStep;
			
			// evaluate data points for y axis to the left
			qvY1Axis << pinfo->getData( iDataChosen );
			if( iDataChosen==GRB_TEMP ) {
				qvTempMax << pinfo->getData( GRB_TMAX );
				qvTempMin << pinfo->getData( GRB_TMIN );
			}
			
			// evaluate data points for y axis to the right, currently fixed to wind direction
			// and derivative of wind direction
			dDirH = pinfo->windDir;
			qvWindDir << pinfo->windDir;
			
			// avoid peak in derivative of first data point
			if( bFirst ) { 
				bFirst = false;
				qvWindDev << 0.;
			} else {
				// assume that change of wind direction is always the smaller angle 
				if( abs( dDir - dDirH ) > 180.0 ) {
					qvWindDev << ( (360 - abs(dDir - dDirH)) / dTimeStep );
				} else {
					qvWindDev << ( abs(dDir - dDirH) / dTimeStep );
				}
			}
			dDir = dDirH;
			
			delete pinfo;
		} else {
			break;
		}

		// get next data point on path between start and end point		
		myOrtho->getCoordsForDist(px, py, dDistStep, myAngle, &tmpLon, &tmpLat );
		myOrtho->setPoints( tmpLon, tmpLat, lonEnd, latEnd );
		myAngle = myOrtho->getAzimutDeg();
		px = tmpLon;
		py = tmpLat;

		// get next date in 1 hour steps
		dateTimeStep += (time_t)(dTimeStep * 3600.);
		dateDataStep = reader->getClosestDateFromDate( dateTimeStep );
	}

	delete myOrtho;
	delete myBoatSpeed;
	
	// set initial display
	qvXAxis  = &qvHoursFromNow;
	qvY2Axis = &qvWindDev;
	return true;
}

//-------------------------------------------------------------------------------
// init all data strcture for plotting
//-------------------------------------------------------------------------------
void CurveDrawer::initDataPlot( const int &iDataChosen )
{
	qwtDataPlot->setAutoReplot( false );


	//------------------------------------------------------------------	
	// set up x-axis
	//
	double dTimeStep = getXAxisStep( (*qvXAxis).first(), (*qvXAxis).last());
	qwtDataPlot->setAxisTitle( QwtPlot::xBottom, tr("hours from now") );
	// check whether historical data is loaded
	if( (tCurDate - tRefDate) < (time_t)604800 )
	{
		chkCurrent->setChecked( true );
		slotCurrentClicked( true );
		
	} else {
		chkCurrent->setChecked( false );
		slotCurrentClicked( false );
	}

	// set up top x-axis
	// plot distances on top axis only for waypoint plot (two points given in constructor)
	if( lonEnd < 999 ) {
		qwtDataPlot->enableAxis( QwtPlot::xTop, true );	
		qwtDataPlot->setAxisTitle( QwtPlot::xTop, tr("distance [nm]") );
		qwtDataPlot->setAxisScaleDraw(QwtPlot::xTop, new CustomQwtScale( *qvXAxis, qvDist ) );
		qwtDataPlot->setAxisScale( QwtPlot::xTop, (*qvXAxis).first(), (*qvXAxis).last(), dTimeStep );		
		
	} else {
		qwtDataPlot->enableAxis( QwtPlot::xTop, false );
	}
	
	// set up left y-axis
	qwtDataPlot->setAxisTitle( QwtPlot::yLeft, getPlotUnit( iDataChosen ) );
	qwtDataPlot->setAxisScaleEngine( QwtPlot::yLeft, qwtYScale[0] );
	qwtDataPlot->setAxisAutoScale( QwtPlot::yLeft );
	
	// set up right y-axis
	qwtDataPlot->setAxisTitle( QwtPlot::yRight, tr("degree") );
	qwtDataPlot->enableAxis( QwtPlot::yRight, true );
	qwtDataPlot->setAxisScale( QwtPlot::yRight, 0., 360.1, 30.  );
	//	qwtDataPlot->setAxisScaleEngine( QwtPlot::yRight, qwtYScale[1] );
	qwtDataPlot->setAxisAutoScale( QwtPlot::yRight );
		
	//------------------------------------------------------------------
	// set up plot for bottom left plot
	//
	qwtCurve[0]->setPen( QPen( QColor(0,0,255,127) ) );
	qwtCurve[0]->setStyle( QwtPlotCurve::Lines );
	//	qwtCurve[0]->setCurveAttribute( QwtPlotCurve::Fitted );
	qwtCurve[0]->setData( *qvXAxis, qvY1Axis );
	qwtCurve[0]->attach( qwtDataPlot );

	//------------------------------------------------------------------	
	// set up plot for top right plot
	//
	qwtCurve[1]->setAxis( QwtPlot::xBottom, QwtPlot::yRight );
	qwtCurve[1]->setPen( QPen( QColor(32,32,0,127) ) );
	qwtCurve[1]->setBrush( QBrush( QColor(32,32,32,20), Qt::SolidPattern ) );
	qwtCurve[1]->setBaseline(0.);
	qwtCurve[1]->setStyle( QwtPlotCurve::Steps );
	//	qwtCurve[1]->setCurveAttribute( QwtPlotCurve::Fitted );
		
	// plot distances in waypoint plot (two points given in constructor)	
	if( lonEnd < 999 ) {
		qwtCurve[1]->setData( qvDist, *qvY2Axis );
	} else {
		qwtCurve[1]->setData( *qvXAxis, *qvY2Axis );
	}
	qwtCurve[1]->attach( qwtDataPlot );
	
	//------------------------------------------------------------------
	// set up marker and picker in plot
	//
	// draw marker line at current date
	QDateTime dt = Util::applyTimeZone( reader->getClosestDateFromNow() );
	QPen myPen = QPen( QColor(2,2,0,10) );
	myPen.setDashOffset( 1. );
	qwtMarker[0]->setLineStyle( QwtPlotMarker::VLine );
	qwtMarker[0]->setLinePen( QPen( Qt::DashDotDotLine ) );
	qwtMarker[0]->setXValue(0);

	QString sPos = dt.toString( "dd.MM. hh:mm\n" );
	sPos += Util::formatLongitude( lonStart ).replace("'", "''");
	sPos += "\n";
	sPos += Util::formatLatitude( latStart ).replace("'", "''");

	qwtMarker[0]->setLabel( sPos );
	qwtMarker[0]->setLabelAlignment( Qt::AlignTop | Qt::AlignRight );
	qwtMarker[0]->attach( qwtDataPlot );

	// draw marker line at end point
	if( lonEnd < 999 ) {
		sPos = "\n";
		sPos += Util::formatLongitude( lonEnd ).replace("'", "''");
		sPos += "\n";
		sPos += Util::formatLatitude( latEnd ).replace("'", "''");
		qwtMarker[1]->setLineStyle( QwtPlotMarker::VLine );
		qwtMarker[1]->setLinePen( QPen() );
		qwtMarker[1]->setXValue( qvHoursFromNow.last() );
		qwtMarker[1]->setLabel( sPos );
		qwtMarker[1]->setLabelAlignment( Qt::AlignTop | Qt::AlignLeft );
		qwtMarker[1]->attach( qwtDataPlot );
	}
	
	// setup picker
	qwtPicker = new CustomQwtPicker( QwtPlot::xBottom, QwtPlot::yLeft,
									 QwtPicker::PointSelection | QwtPicker::DragSelection,
									 QwtPlotPicker::CrossRubberBand, QwtPicker::ActiveOnly, 
									 qwtDataPlot->canvas(), dt );
    qwtPicker->setRubberBand(QwtPicker::CrossRubberBand);
	qwtPicker->setRubberBandPen(QColor(64,64,124,255));
    qwtPicker->setTrackerPen(QColor(0, 0, 255, 127));
	qwtPicker->setEnabled(true);
	
	// connect signals
//	connect(qwtPicker, SIGNAL(moved(const QwtDoublePoint &)), qwtPicker, SLOT(moved(const QwtDoublePoint &)));
//	connect(qwtPicker, SIGNAL(selected(const QwtDoublePoint &)), qwtPicker, SLOT(selected(const QwtDoublePoint &)));
	
	
	//------------------------------------------------------------------
	// set up legend, activate curves
	//
	qwtLegend->setItemMode( QwtLegend::CheckableItem );
	qwtDataPlot->insertLegend( qwtLegend, QwtPlot::BottomLegend );
	activateLegend( qwtCurve[0], true );
	activateLegend( qwtCurve[1], false );	

	qwtDataPlot->replot();
}

//-------------------------------------------------------------------------------
// Create data plot
// todo: method must be redone
//-------------------------------------------------------------------------------
void CurveDrawer::createDataPlot( const int &iDataChosen, const QString &sTitle )
{
	QString sUnitYLeft;
	
	qwtCurve[0]->setData( *qvXAxis, qvY1Axis );
	qwtCurve[0]->itemChanged();
	qwtCurve[0]->setTitle( sTitle );
	sUnitYLeft = getPlotUnit( iDataChosen );
	qwtDataPlot->setAxisTitle( QwtPlot::yLeft, sUnitYLeft );
	activateLegend( qwtCurve[0], true );
	
	// add min and max curve for temperature plot
	if( iDataChosen == GRB_TEMP ) {
		qwtCurve[2]->setAxis( QwtPlot::xBottom, QwtPlot::yLeft );
		qwtCurve[2]->setPen( QPen( QBrush( QColor(132,0,132,132), Qt::SolidPattern ), 1.4, 
							 Qt::SolidLine, Qt::SquareCap, Qt::BevelJoin ) );
		qwtCurve[2]->setStyle( QwtPlotCurve::Steps );
		qwtCurve[2]->setData( *qvXAxis, qvTempMin );
		qwtCurve[2]->attach( qwtDataPlot );
		
		qwtCurve[3]->setAxis( QwtPlot::xBottom, QwtPlot::yLeft );
		qwtCurve[3]->setPen( QPen( QBrush( QColor(0,132,0,132), Qt::SolidPattern ), 1.4,
							 Qt::SolidLine, Qt::SquareCap, Qt::BevelJoin ) );
		qwtCurve[3]->setStyle( QwtPlotCurve::Steps );
		qwtCurve[3]->setData( *qvXAxis, qvTempMax );
		qwtCurve[3]->attach( qwtDataPlot );
		
		activateLegend( qwtCurve[2], true );
		activateLegend( qwtCurve[3], true );
	} else {
		qwtCurve[2]->detach();
		qwtCurve[3]->detach();
	}

	qwtDataPlot->replot();
}

//-------------------------------------------------------------------------------
// Initialize interface by registring signals and call of show
//-------------------------------------------------------------------------------
void CurveDrawer::initInterface()
{
	connect( btExit,		SIGNAL(clicked()),		this, SLOT(slotExitClicked()) );
	connect( chkCurrent,	SIGNAL(clicked(bool)),	this, SLOT(slotCurrentClicked(bool)) );	
	connect( cbData,		SIGNAL(activated(int)),	this, SLOT(slotDataChosen(int)) );
	connect( qwtDataPlot,	SIGNAL(legendChecked( QwtPlotItem *, bool ) ), this,
							SLOT( slotLegendChecked( QwtPlotItem *, bool ) ) );
	setModal( true );
	show();
}
//-------------------------------------------------------------------------------
//-------------------------------------------------------------------------------
// slots for button events
void CurveDrawer::slotExitClicked()		{ delete this; }

//-------------------------------------------------------------------------------
// Change from current time to date display on x-axis
//-------------------------------------------------------------------------------
void CurveDrawer::slotCurrentClicked( bool bState )
{
	if (bState) {
		// display graph starting from current time
		double dTimeStep = getXAxisStep( (*qvXAxis).first(), (*qvXAxis).last());
		qwtDataPlot->setAxisScale( QwtPlot::xBottom, (*qvXAxis).first(), (*qvXAxis).last(), dTimeStep );		
		qwtDataPlot->setAxisScaleDraw(QwtPlot::xBottom, new QwtScaleDraw() );
	} else {
		// display graph with historical data (x-axis in date format)
		qwtDataPlot->setAxisScale( QwtPlot::xBottom, (*qvXAxis).first(), (*qvXAxis).last(), 24. );			
		qwtDataPlot->setAxisScaleDraw(QwtPlot::xBottom, new CustomQwtScale( *qvXAxis, qvDates ) );
	}
	qwtDataPlot->replot();	
}
//-------------------------------------------------------------------------------
// slot for new data is chosen from drop-down box
//-------------------------------------------------------------------------------
void CurveDrawer::slotDataChosen( int iDataChosen )
{
	int iData;
	QString sTitle;
	if( iDataChosen==0 ) {
		iData = cbData->itemData( cbData->currentIndex() ).toInt();
		sTitle = cbData->itemText( cbData->currentIndex() );
	} else {
		iData = cbData->itemData( iDataChosen ).toInt();
		sTitle = cbData->itemText( iDataChosen );
	}
	
	createTable( iData );
	createDataPlot( iData, sTitle );
}
//-------------------------------------------------------------------------------
// slot for legend item checking
// toggle legend item check box and do replot
//-------------------------------------------------------------------------------
void CurveDrawer::slotLegendChecked( QwtPlotItem *pItem, bool on )
{
	activateLegend( pItem, on );
	qwtDataPlot->replot();	
}
//-------------------------------------------------------------------------------
// check / decheck legend without a replot
//-------------------------------------------------------------------------------
void CurveDrawer::activateLegend( QwtPlotItem *pItem, bool on )
{
	// switch between wind direction and derivateive of direction for Y2 axis
	if( pItem == qwtCurve[1] ) {
		if( qvY2Axis == &qvWindDir ) {
			qvY2Axis = &qvWindDev;
			qwtCurve[1]->setTitle( tr("Wind Deviation") );
			qwtDataPlot->setAxisTitle( QwtPlot::yRight, tr("deg/h") );
			qwtDataPlot->setAxisAutoScale( QwtPlot::yRight );
		} else {
			qvY2Axis = &qvWindDir;
			qwtCurve[1]->setTitle( tr("Wind Direction") );
			qwtDataPlot->setAxisTitle( QwtPlot::yRight, tr("degree") );
			qwtDataPlot->setAxisScale( QwtPlot::yRight, 0., 360.1, 30.  );
			qwtDataPlot->setAxisScaleEngine( QwtPlot::yRight, NULL );
		}
		qwtCurve[1]->setData( *qvXAxis, *qvY2Axis );
		on = false;
	// toggle legend item check box
	} else {
		pItem->setVisible( on );
	}
	// check legend button
	QWidget *qWid = qwtDataPlot->legend()->find( pItem );
	if( qWid && qWid->inherits( "QwtLegendItem" ) )
		((QwtLegendItem *)qWid)->setChecked( on );
	
}
//-------------------------------------------------------------------------------
// get data unit for axis title
//-------------------------------------------------------------------------------
QString CurveDrawer::getPlotUnit( const int &iDataChosen )
{
	QString sUnit;
	switch( iDataChosen ) {
		case GRB_WIND_XY2D:
			sUnit = Util::getSetting( "unitsWindSpeed", "" ).toString();
			break;
		case GRB_TEMP:
			sUnit = Util::getSetting( "unitsTemp", "" ).toString();
			break;
		case GRB_PRESSURE_MSL:
			sUnit = "hPa"; 
			break;
		case GRB_CLOUD_TOT:
			sUnit = tr("Percent [%]");
			break;
		case GRB_PRECIP_RATE:
			sUnit = tr("mm/h");
			break;
		case GRB_PRECIP_TOT:
			sUnit = tr("mm/h");
			break;			
		default:
			sUnit = "";
	}	
	return sUnit;
}
//-------------------------------------------------------------------------------
// additional constructor
//-------------------------------------------------------------------------------
bool CurveDrawer::privateConstructor()
{
	setupUi(this);
	setWindowTitle(tr("Plot data of chosen point"));
	
    // checks for GRIB data
    if( !plotter->isReaderOk() ) {
        QMessageBox::critical (this, tr("Error"),tr("Can't create dataplot:\n\nNo GRIB file opened."));
		delete this;
		return false;
    }
	reader = plotter->getReader();
	if( reader == NULL ) {
		QMessageBox::critical (this, tr("Error"),tr("Can't create dataplot:\n\nNo GRIB file opened."));
		delete this;
		return false;	
	}
    GriddedRecord *record = reader->getFirstRecord();
	if( record == NULL ) {
        QMessageBox::critical (this, tr("Error"),tr("Can't create dataplot\n\nGRIB area undefined."));
		delete this;
		return false;
    }
	if( !record->isPointInMap(lonStart, latStart) ) {
		QMessageBox::critical (this, tr("Error"),tr("Can't create dataplot:\n\nPoint outside GRIB area."));
		delete this;
		return false;
	}
	
	// get meatadata of GRIB data
	tRefDate = record->getRecordRefDate();
	tCurDate = time(NULL);
	
	//----------------------------------------------
	// Constructor of for all used objects
	qwtCurve[0]  = new QwtPlotCurve( tr("Wind Speed") );
	qwtCurve[1]	 = new QwtPlotCurve( tr("Wind Direction") );
	qwtCurve[2]  = new QwtPlotCurve( tr("Temp Min") );
	qwtCurve[3]  = new QwtPlotCurve( tr("Temp Max") );	
	qwtYScale[0] = new QwtLinearScaleEngine();
	qwtYScale[1] = new QwtLinearScaleEngine();
	qwtMarker[0] = new QwtPlotMarker();
	qwtMarker[1] = new QwtPlotMarker();	
	qwtLegend	 = new QwtLegend();
	
	// list available data types ion combo box
	if( reader->hasDataType( GRB_WIND_XY2D ) )		{ cbData->addItem(tr("Wind"), GRB_WIND_XY2D ); };
	if( reader->hasDataType( GRB_PRESSURE_MSL ) )	{ cbData->addItem(tr("Pressure"), GRB_PRESSURE_MSL ); };
	if( reader->hasDataType( GRB_TEMP ) )			{ cbData->addItem(tr("Temperature"), GRB_TEMP ); };
	if( reader->hasDataType( GRB_CLOUD_TOT ) )		{ cbData->addItem(tr("Cloud cover"), GRB_CLOUD_TOT ); };
	if( reader->hasDataType( GRB_PRECIP_RATE ))		{ cbData->addItem(tr("Percipitation Rate"), GRB_PRECIP_RATE ); };
	if( reader->hasDataType( GRB_PRECIP_TOT ) )		{ cbData->addItem(tr("Percipitation Total"), GRB_PRECIP_TOT ); };
	
	return true;
}

//--------------------------------------------------------------------------------------------
// get step size for major tics of axis
//--------------------------------------------------------------------------------------------
double CurveDrawer::getXAxisStep( const double &dFirst, const double &dLast )
{
	if( abs(dFirst - dLast) > 30 ) {
		return 12.;
	} else {
		return 3.;
	}	
}